/************************************************************************
 *                                                                      *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ************************************************************************/

#include "rep.h"

struct offset_str *
diff_files(int fd_orig, int fd_crack)
{
    unsigned char *mem_orig,	/* memory map of orig file */
                  *mem_crack;	/* memory map of closed file */
    extern char verbose;
                  
    size_t i, file_len;
    offset *start, *end, *node;
    
    /* we use a linked list of offset structs to contain
     * all the bytes and offsets that need to be patched
     */
      node = start = end = NULL;


    file_len = file_size(fd_orig);

    /* read into memory */
    if ( (mem_orig=memorize_file(fd_orig)) == NULL )
	fatal("[error] > compare_files::memorize_files()");
    if ( (mem_crack=memorize_file(fd_crack)) == NULL )
	fatal("[error] > compare_files::memorize_files()");

    /* a while loop would break on 0x00 so we need to iterate... grumble */
    for (i=0; i<file_len;i++) {
	if (mem_orig[i] == mem_crack[i])
	    continue;
	else {
	    if ( (node = (offset *) malloc(sizeof(offset))) == NULL) {
	      return(NULL);
	    }
	    if (!start) {
		    start = node;
	    }

	    memset(node, 0, sizeof(struct offset_str));

	    node->offset = i;
	    node->byte = mem_crack[i];
	    node->next = NULL;
	    if (end)
	       end->next = node;
	    end = node;
	}
    }

    if (verbose) {
        node = start;
        while (node) {
       	    printf("offset: 0x%X\n", node->offset);
       	    printf("byte  : 0x%X\n", node->byte);

	    node = node->next;
        }
    }

    /* memory manager is I */
    free(mem_orig);
    free(mem_crack);

    return(start);
}

/* free the offset linked list... */
void
free_offsets(struct offset_str *start)
{
    struct offset_str *node, *last;
    last = start;

    while(last) {
	node = last->next;
	free(last);

	last = node;
    }
}

/* create the patch arrays */
int *
create_offset_array(struct offset_str *start)
{
    unsigned int *offsets;
    struct offset_str *node;
    int length, i = 0;

    node = start;
    length = list_length( node );

    offsets = malloc(length * sizeof(int));

    while(node) {
	offsets[i] = node->offset;
	i++;
	node = node->next;
    }
	
    return(offsets);
}

char *
create_byte_array(struct offset_str *start)
{
    unsigned char *byte;
    struct offset_str *node;
    int length, i = 0;

    node = start;
    length = list_length( node );

    byte = malloc(length);

    while(node) {
	byte[i] = node->byte;
	i++;
	node = node->next;
    }

    return(byte);
}

/* get the size of the file */
size_t
file_size(int fd)
{
    struct stat file_stat;

    if ( (fstat(fd, &file_stat)) < 0)
	fatal("[error] > file_size::fstat()");

    return( (size_t) file_stat.st_size );
}

/* get the length of the linked list */
int
list_length(struct offset_str *list)
{
    struct offset_str *node;
    int i=0;

    node = list;

    while(node) {
	i++;
	node = node->next;
    }
    return(i);
}
		    
/* read file into memory */
unsigned char *
memorize_file(int fd)
{
    unsigned char *mem_file, *mem_ptr;
    size_t size_of_file, c;

    if ((size_of_file = file_size(fd)) == -1)
	fatal("[error] > memorize_file::file_size()");

    /* initialize our memory */
    if ((mem_file = (unsigned char *) malloc(size_of_file)) == NULL)
	fatal("[error] > memorize_file::malloc()");

    memset( mem_file, 0, size_of_file );
    mem_ptr = mem_file;

    /* read in the bytes of the file */
    while ((c = read(fd, mem_ptr, BUFSIZ)) != 0) {
	switch (c) {
	    case -1:
		perror("[warning] > memorize_file::read()");
		return(NULL);
		break;		/* not reached */
	    default:
		mem_ptr += c; 	/* point to the end of new bytes */
		break;
	}
    }
    return(mem_file);
}

/* check if files are identical sizes */
/* macro? 
 * #define CHECK_FILES ((file_size(a) == file_size(b))? 0 :1) */
int
check_files(int a, int b)
{
     if (file_size(a) == file_size(b))
	return(0);
     else
	return(1);
}
