/************************************************************************
 *                                                                      *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ************************************************************************/


#include "rep.h"

char verbose = 0;

int
main(int argc, char** argv)
{
    int fd_orig, fd_crack, fd_out, c;
    char *orig_file = NULL,	/* original file name */
         *crack_file = NULL,	/* cracked file name */
	 *out_file = NULL;	/* output file name */
    extern char verbose;

    struct offset_str *differences;

    extern char *optarg;		/* getopt stuff */
    extern int optind, opterr, optopt;	/* getopt stuff */

    if (argc < 2)
	usage(argv[0]);

    while ((c = getopt(argc, argv, "o:c:w:vV")) != EOF) {
        switch (c) {
	    case 'o':
		orig_file = optarg;
		break;
	    case 'c':
		crack_file = optarg;
		break;
	    case 'w':
		out_file = optarg;
		break;
	    case 'v':
		verbose = 1;
		break;
	    case'V':
		printf(".oO( %s Ver. %s )Oo.\n", argv[0], REP_VERSION);
		/* fall through to usage() */
	    default:
		usage(argv[0]);
		break;		/* never reached */
	}
    }

    if (!orig_file || !crack_file)
	usage(argv[0]);	/* will exit */

    /* Open all the files we need.  */
    if ((fd_orig = open(orig_file, O_RDONLY)) == -1)
        fatal("[error] > main::open( orig_file )");
    if ((fd_crack = open(crack_file, O_RDONLY)) == -1)
	fatal("[error] > main::open( crack_file )");

    if (!out_file)
	fd_out = 1;
    else {
	if ((fd_out = open(out_file, O_CREAT|O_WRONLY, 0644)) == -1)
	    fatal("[error] > main::open( out_file )");
    }

    if(1) /* XXX use optarg to determine behaviour */
	if( check_files( fd_orig, fd_crack ))
	    fatal("[error] > Files differ");
    
     differences = diff_files(fd_orig, fd_crack);

     make_patch( differences, fd_out);


    return(0);
}

/* patch wrapper */
int
make_patch( struct offset_str *list, int out_file )
{
    unsigned int *offsets;
    unsigned char *bytes;
    int length, i, arr_len;

    char *off_arr_str, *str_p;

    
    length = list_length( list );
    arr_len = 20 + (10 * length);
    arr_len = arr_len << 1;

    offsets = create_offset_array( list );
    bytes = create_byte_array( list );

    off_arr_str = (char *)  malloc(arr_len);
    str_p = off_arr_str;

    write_patch_head(out_file);
    sprintf(off_arr_str, "%d;\n\tint offsets[] = {", length);
    str_p += strlen(off_arr_str);

    for (i=0; i<length;i++) {
	sprintf(str_p," %d,", offsets[i]);
	/* advance the pointer to our next location... */
	str_p = off_arr_str;
	str_p += strlen(off_arr_str);
    }
    if (str_p != off_arr_str) {
	*(str_p - 1) = ' ';
    }

    sprintf(str_p, "};\n\tunsigned char bytes[] = {");
    str_p = off_arr_str;
    str_p += strlen(off_arr_str);

    for (i=0; i<length; i++) {
	sprintf(str_p, " %d,", bytes[i]);
	/* advance the pointer to our next location... */
	str_p = off_arr_str;
	str_p += strlen(off_arr_str);
    }
    if (str_p != off_arr_str) {
	*(str_p - 1) = ' ';
    }

    sprintf(str_p, "};\n\n");
    write(out_file, off_arr_str, strlen(off_arr_str));
    write_patch_foot(out_file);

    return(0);
}

int
write_patch_head(int fd)
{
    char *head =
	    "#include <stdio.h>\n\n"
	    "int main(int argc, char** argv)\n"
	    "{\n"
	    "\tFILE *out;\n"
	    "\tint i, length = ";

    write(fd, head, strlen(head));
    return(strlen(head));
}

void
write_patch_foot(int fd)
{
    char *foot = 
	    "\tif (argc != 2) {\n"
	    "\t\tfprintf(stderr, \"%s <file to patch>\\n\", argv[0]);\n"
	    "\t\texit(1);\n"
	    "\t}\n\n"
	    "\tif ( (out = fopen(argv[1], \"r+\")) == NULL) {\n"
	    "\t\tperror(\"fopen failed\");\n"
	    "\t\texit(1);\n"
	    "\t}\n"
	    "\tfor (i=0; i<length; i++) {\n"
	    "\t\tfseek(out, offsets[i], SEEK_SET);\n"
	    "\t\tfputc(bytes[i], out);\n"
	    "\t}\n\n"
	    "\treturn(0);\n"
	    "}\n";
    write(fd, foot, strlen(foot));
}

/* */


/* Maintance functions */

int
fatal(const char *s)
{
    perror(s);
    exit( EXIT_FAILURE );
}

int
usage(const char *prog_name)
{
    printf("\n%s <-vV> [-o orig file] [-c crack file] <-w out file>\n\n", prog_name);
    printf("\t-o	original/unpatched file\n");
    printf("\t-c	cracked/patched file\n");
    printf("\t-w	crack out file\n");
    printf("\t-v	verbose out put\n");
    printf("\t-V	Version\n");
    printf("\n");

    exit( EXIT_SUCCESS );
}
